program sst_pert_fbtr 

! Randomly pick up several SST data from sst_climo and perturb the upper 150m
! of initial ocean condition for Princeton Ocean Model

use netcdf
implicit none
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 integer, parameter              :: im = 869, jm = 449, nl = 33
 integer                         :: pertid
 integer                         :: i, j, k, nh
 integer                         :: ncIDc, ncIDg, ncIDi, ncIDcli, var_id
 integer                         :: d, t 
 integer                         :: startdate, iseed, imonth, iday, ihour, ntime
 integer                         :: n, ptime
 real, parameter                 :: climo_sst_std = 0.5
 real                            :: pscale, ran1
 real, dimension(im,jm)          :: xland, hh
 real, dimension(im,jm)          :: sstm, sstv, sstin, sstpert 
 real, allocatable               :: zz(:), tt(:), area(:)
 real, allocatable               :: tclim(:,:,:)
 real, allocatable               :: ocean_t_in(:,:,:), ocean_t_clim(:,:,:)
 real, allocatable               :: ocean_t(:,:,:,:), sstcl(:,:,:)
 character(len=100)              :: gridfile, tsinfile, tsclimfile, stormname
 character(len=80)               :: dim_name
 character(len=19), parameter    :: climofile     = "sst_clim.nc"
 character(len=10)               :: startdatestr 

 read(5,*) stormname 
 write(6,*) trim(adjustl(stormname))
 write(gridfile,'(a,''.grid.nc'')')       trim(adjustl(stormname))
 write(tsinfile,'(a,''.ts_initial.nc'')') trim(adjustl(stormname))
 write(tsclimfile,'(a,''.ts_clim.nc'')')  trim(adjustl(stormname))

 read(5,*)  startdate
 write(6,*) startdate
 read(5,*)  pertid 
 write(6,*) pertid 

 write(startdatestr,'(i10)') (startdate + pertid)

 read(startdatestr(5:6),  fmt='(i2)') imonth
 read(startdatestr(7:8),  fmt='(i2)') iday
 read(startdatestr(9:10), fmt='(i2)') ihour
 iseed = -(imonth * 10000 + iday * 100 + ihour)
 print*,"iseed:",iseed

! Read ocean temperature from ocean_clim_xx.nc  

 call nc_check(nf90_open(climofile, nf90_nowrite, ncIDc), & 
                "sst_pert", "open " // climofile)
 call nc_check(nf90_inq_dimid(ncIDc, "time", var_id), &
                "sst_pert", "inq_dimid Time in " // climofile)
 call nc_check(nf90_inquire_dimension(ncIDc, var_id, dim_name, ntime), &
                "sst_pert", "inquire_dimension Time in " // climofile )
 call nc_check(nf90_inq_dimid(ncIDc, "z", var_id), &
                "sst_pert", "inq_dimid vertical in " //climofile )
 call nc_check(nf90_inquire_dimension(ncIDc, var_id, dim_name, nh), &
                "sst_pert", "inquire_dimension Time in " // climofile )
 
 allocate (ocean_t(im,jm,nh,ntime),sstcl(im,jm,ntime))
 allocate (zz(nl), tt(nl), area(nl))
 allocate (ocean_t_in(im,jm,nl), ocean_t_clim(im,jm,nl), tclim(im,jm,nl))
 
 call nc_check( nf90_inq_varid(ncIDc, "t", var_id), &
                "sst_pert", "inq_varid t in " // climofile )
 call nc_check( nf90_get_var(ncIDc, var_id, ocean_t), &
                "sst_pert", "get_var t in " // climofile )
 call nc_check( nf90_close(ncIDc), "sst_pert", "close " // climofile )

! Read land-sea mask information from grid.nc file
 
 call nc_check( nf90_open(gridfile, nf90_nowrite, ncIDg), &
                "sst_pert", "open pom_grid" )
 call nc_check( nf90_inq_varid(ncIDg, "fsm", var_id), &
                "sst_pert", "inq_varid fsm in pom.grid file" )
 call nc_check( nf90_get_var(ncIDg, var_id, xland), &
                "sst_pert", "get_var fsm in pom.grid file" )
 call nc_check( nf90_inq_varid(ncIDg, "h", var_id), &
                "sst_pert", "inq_varid h in pom.grid file" )
 call nc_check( nf90_get_var(ncIDg, var_id, hh), &
                "sst_pert", "get_var h in pom.grid file" )
 call nc_check( nf90_close(ncIDg), "sst_pert", "close pom.grid" )

 sstcl(:,:,:) = ocean_t(:,:,1,:) ! Get SST from climofile

! Determine the random time from the file to use

  ptime = min(max(1, nint(ran1(iseed)*real(ntime+1)-0.5)), ntime)
  print*,"iseed, ptime",iseed,ptime

 sstm(:,:) = 0.0  ;  sstv(:,:) = 0.0

 !Start generating SST perturbation
 do i = 1, im  
   do j = 1, jm

     if ( xland(i,j) == 1 ) then       ! xland == 1 represents water
       sstm(i,j)    = sum(sstcl(i,j,:)) / real(ntime)
       sstcl(i,j,:) = sstcl(i,j,:) - sstm(i,j)
       sstv(i,j)    = sqrt( sum(sstcl(i,j,:)*sstcl(i,j,:)) / real(ntime-1) )
       if ( sstv(i,j) == 0. ) then
         sstv(i,j) = 0.1
       end if
         pscale = climo_sst_std / sstv(i,j)
         ! Using perturbation at ptime
         sstpert(i,j) = sstcl(i,j,ptime) * pscale 
     else
         sstpert(i,j) = 0.0
     end if

   end do 
 end do
  print*,"ptime:",ptime
  print*,"pscale:",pscale

 !Read ocean temperature from storm.ts_initial.nc file

 call nc_check( nf90_open(tsinfile, nf90_write, ncIDi), &
                "sst_pert", "open pom.ts_initial" )
 call nc_check( nf90_inq_varid(ncIDi, "t", var_id), &
                "sst_pert", "inq_varid ocean temperature" )
 call nc_check( nf90_get_var(ncIDi, var_id, ocean_t_in), &
                "sst_pert", "get_var ocean temperature" )
 call nc_check( nf90_inq_varid(ncIDi, "z", var_id), &
                "sst_pert", "inq_varid z" )
 call nc_check( nf90_get_var(ncIDi, var_id, zz), &
                "sst_pert", "get_var z" )

 sstin(:,:) = ocean_t_in(:,:,1)

 do i = 1, im  
   do j = 1, jm
     if ( xland(i,j) == 1 ) then
       sstin(i,j) = sstin(i,j) + sstpert(i,j) ! Perturb SST in ts_initial
     end if
   end do  
 end do

! Mix SST perturbation into upper ocean (150m)

 call mixsstz(ocean_t_in,sstin,hh,xland,zz,im,jm,nl,0)

! Put perturbed SST back to ts_initial file

 call nc_check( nf90_inq_varid(ncIDi, "t", var_id), &
                "sst_pert", "inq_varid t in ts_initial" )
 call nc_check( nf90_put_var(ncIDi, var_id, ocean_t_in), &
                "sst_pert", "put_var t back into ts_initial" )
 call nc_check( nf90_close(ncIDi), "pom.ts_initial", "close pom.ts_initial.nc" )

! Calculate tclim after perturbation

 do k = 1,nl 
   tt(k) = 0.
   area(k) = 0.
   do i = 1,im
     do j = 1,jm
       if (ocean_t_in(i,j,k) /= 0.) then
         tt(k) = tt(k)+ocean_t_in(i,j,k)
         area(k) = area(k)+1.
       end if
     end do
   end do
     tt(k) = tt(k)/area(k)
 end do
 do k = 1,nl
   do i = 1,im
     do j = 1,jm
       tclim(i,j,k) = tt(k)
     end do
   end do
 end do

! Put tclim back to stormname.ts_clim.nc

 call nc_check( nf90_open(tsclimfile, nf90_write, ncIDcli), &
                "sst_pert", "open ts_clim" )
 call nc_check( nf90_inq_varid(ncIDcli, "t", var_id), &
                "sst_pert", "inq_varid t in ts_clim" )
 call nc_check( nf90_get_var(ncIDcli, var_id, ocean_t_clim), &
                "sst_pert", "get_var t in ts_clim" )
 call nc_check( nf90_put_var(ncIDcli, var_id, tclim), &
                "sst_pert", "put_var t in ts_clim" )
 call nc_check( nf90_close(ncIDcli), "ts_clim", "close ts_clim.nc" )

 print*,"ocean_t_clim(81,205,1)",ocean_t_clim(81,205,1)
 print*,"tclim(81,205,1)",tclim(81,205,1)
 deallocate(ocean_t, sstcl)
 deallocate(ocean_t_in, ocean_t_clim, tclim)

end program sst_pert_fbtr 

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
subroutine nc_check(istatus, subr_name, context)
use netcdf
implicit none
  integer,          intent (in)              :: istatus
  character(len=*), intent(in)               :: subr_name
  character(len=*), intent(in), optional     :: context
  character(len=129)                         :: error_msg

!  if no error, nothing to do here.  we are done.
  if( istatus == nf90_noerr) return
!  something wrong.  construct an error string
  if (present(context) ) then
    error_msg = trim(context) // ': ' // trim(nf90_strerror(istatus))
  else
    error_msg = nf90_strerror(istatus)
  endif
  write(6,*) error_msg
  stop
end subroutine
FUNCTION ran1(idum)

 INTEGER idum,IA,IM,IQ,IR,NTAB,NDIV
 REAL ran1,AM,EPS,RNMX
 PARAMETER (IA=16807,IM=2147483647,AM=1./IM,IQ=127773,IR=2836,&
           NTAB=32,NDIV=1+(IM-1)/NTAB,EPS=1.2e-7,RNMX=1.-EPS)
 INTEGER j,k,iv(NTAB),iy
 SAVE iv,iy
 DATA iv /NTAB*0/, iy /0/

 if (idum.le.0.or.iy.eq.0) then
   idum=max(-idum,1)
   do j=NTAB+8,1,-1
     k=idum/IQ
     idum=IA*(idum-k*IQ)-IR*k
    if (idum.lt.0) idum=idum+IM
    if (j.le.NTAB) iv(j)=idum
   end do
   iy=iv(1)
 endif

 k=idum/IQ
 idum=IA*(idum-k*IQ)-IR*k
 if (idum.lt.0) idum=idum+IM
  j=1+iy/NDIV
  iy=iv(j)
  iv(j)=idum
  ran1=min(AM*iy,RNMX)

 return
end function
